import { Path, FilterDefinitions, OrderDefinition } from '@supabase-cache-helpers/postgrest-filter';
import { QueryWithoutWildcard, AnyPostgrestResponse, PostgrestPaginationResponse, PostgrestHasMorePaginationResponse } from '@supabase-cache-helpers/postgrest-shared';
import { PostgrestQueryBuilder, PostgrestTransformBuilder } from '@supabase/postgrest-js';
import { GetResult } from '@supabase/postgrest-js/dist/module/select-query-parser';
import { GenericTable, GenericSchema } from '@supabase/postgrest-js/dist/module/types';
import { GenericSchema as GenericSchema$1 } from '@supabase/supabase-js/dist/module/lib/types';

type LoadQueryOps<Q extends string = '*'> = {
    query?: QueryWithoutWildcard<Q> | null;
    disabled?: boolean;
    queriesForTable: () => {
        paths: Path[];
        filters: FilterDefinitions;
    }[];
};
type LoadQueryReturn = {
    selectQuery: string;
    userQueryPaths: Path[] | null;
    paths: Path[];
};
declare const loadQuery: <Q extends string = "*">({ query, disabled, queriesForTable, }: LoadQueryOps<Q>) => LoadQueryReturn | null;

type MutationFetcherResponse<R> = {
    normalizedData: R;
    userQueryData?: R;
};

declare const setFilterValue: (searchParams: URLSearchParams, path: string, op: string, value: string) => void;

type DeleteFetcher<T extends GenericTable, R> = (input: Partial<T['Row']>) => Promise<MutationFetcherResponse<R> | null>;
type DeleteFetcherOptions<S extends GenericSchema, T extends GenericTable, Re = T extends {
    Relationships: infer R;
} ? R : unknown> = Parameters<PostgrestQueryBuilder<S, T, Re>['delete']>[0];
declare const buildDeleteFetcher: <S extends GenericSchema, T extends GenericTable, Re = T extends {
    Relationships: infer R;
} ? R : unknown, Q extends string = "*", R_1 = GetResult<S, T["Row"], Re, Q extends "*" ? "*" : Q>>(qb: PostgrestQueryBuilder<S, T, R_1>, primaryKeys: (keyof T["Row"])[], opts: LoadQueryOps<Q> & {
    count?: "exact" | "planned" | "estimated" | undefined;
}) => DeleteFetcher<T, R_1>;

declare const fetcher: <Result>(q: PromiseLike<AnyPostgrestResponse<Result>>) => Promise<AnyPostgrestResponse<Result>>;

type InsertFetcher<T extends GenericTable, R> = (input: T['Insert'][]) => Promise<MutationFetcherResponse<R>[] | null>;
type InsertFetcherOptions<S extends GenericSchema$1, T extends GenericTable, Re = T extends {
    Relationships: infer R;
} ? R : unknown> = Parameters<PostgrestQueryBuilder<S, T, Re>['insert']>[1];
declare function buildInsertFetcher<S extends GenericSchema$1, T extends GenericTable, Re = T extends {
    Relationships: infer R;
} ? R : unknown, Q extends string = '*', R = GetResult<S, T['Row'], Re, Q extends '*' ? '*' : Q>>(qb: PostgrestQueryBuilder<S, T, Re>, opts: LoadQueryOps<Q> & InsertFetcherOptions<S, T, Re>): InsertFetcher<T, R>;

type PostgrestCursorPaginationFetcher<Type, Args> = (args: Args) => Promise<Type>;
type PostgrestCursorPaginationKeyDecoder<Args> = (args: Args) => {
    cursor?: string;
    order: Pick<OrderDefinition, 'column' | 'ascending' | 'foreignTable'>;
};
declare const createCursorPaginationFetcher: <Schema extends GenericSchema$1, Row extends Record<string, unknown>, Result, Args, Relationships = unknown>(query: PostgrestTransformBuilder<Schema, Row, Result[], Relationships> | null, decode: PostgrestCursorPaginationKeyDecoder<Args>) => PostgrestCursorPaginationFetcher<PostgrestPaginationResponse<Result>, Args> | null;

type PostgrestOffsetPaginationFetcher<Type, Args> = (args: Args) => Promise<Type>;
type PostgrestOffsetPaginationKeyDecoder<Args> = (args: Args) => {
    limit?: number;
    offset?: number;
};
declare const createOffsetPaginationFetcher: <Schema extends GenericSchema$1, Row extends Record<string, unknown>, Result, Args, Relationships = unknown>(query: PostgrestTransformBuilder<Schema, Row, Result[], Relationships> | null, decode: PostgrestOffsetPaginationKeyDecoder<Args>, pageSize: number) => PostgrestOffsetPaginationFetcher<PostgrestPaginationResponse<Result>, Args> | null;
declare const createOffsetPaginationHasMoreFetcher: <Schema extends GenericSchema$1, Row extends Record<string, unknown>, Result, Args>(query: PostgrestTransformBuilder<Schema, Row, Result[], unknown> | null, decode: PostgrestOffsetPaginationKeyDecoder<Args>, pageSize: number) => PostgrestOffsetPaginationFetcher<PostgrestHasMorePaginationResponse<Result>, Args> | null;

type UpdateFetcher<T extends GenericTable, R> = (input: Partial<T['Row']>) => Promise<MutationFetcherResponse<R> | null>;
type UpdateFetcherOptions<S extends GenericSchema, T extends GenericTable, Re = T extends {
    Relationships: infer R;
} ? R : unknown> = Parameters<PostgrestQueryBuilder<S, T, Re>['update']>[1];
declare const buildUpdateFetcher: <S extends GenericSchema, T extends GenericTable, Re = T extends {
    Relationships: infer R;
} ? R : unknown, Q extends string = "*", R_1 = GetResult<S, T["Row"], Re, Q extends "*" ? "*" : Q>>(qb: PostgrestQueryBuilder<S, T, Re>, primaryKeys: (keyof T["Row"])[], opts: LoadQueryOps<Q> & {
    count?: "exact" | "planned" | "estimated" | undefined;
}) => UpdateFetcher<T, R_1>;

type UpsertFetcher<T extends GenericTable, R> = (input: T['Insert'][]) => Promise<MutationFetcherResponse<R>[] | null>;
type UpsertFetcherOptions<S extends GenericSchema$1, T extends GenericTable, Re = T extends {
    Relationships: infer R;
} ? R : unknown> = Parameters<PostgrestQueryBuilder<S, T, Re>['upsert']>[1];
declare const buildUpsertFetcher: <S extends GenericSchema$1, T extends GenericTable, Re = T extends {
    Relationships: infer R;
} ? R : unknown, Q extends string = "*", R_1 = GetResult<S, T["Row"], Re, Q extends "*" ? "*" : Q>>(qb: PostgrestQueryBuilder<S, T, Re>, opts: LoadQueryOps<Q> & {
    onConflict?: string | undefined;
    ignoreDuplicates?: boolean | undefined;
    count?: "exact" | "planned" | "estimated" | undefined;
    defaultToNull?: boolean | undefined;
}) => UpsertFetcher<T, R_1>;

export { DeleteFetcher, DeleteFetcherOptions, InsertFetcher, InsertFetcherOptions, LoadQueryOps, MutationFetcherResponse, PostgrestCursorPaginationFetcher, PostgrestCursorPaginationKeyDecoder, PostgrestOffsetPaginationFetcher, PostgrestOffsetPaginationKeyDecoder, UpdateFetcher, UpdateFetcherOptions, UpsertFetcher, UpsertFetcherOptions, buildDeleteFetcher, buildInsertFetcher, buildUpdateFetcher, buildUpsertFetcher, createCursorPaginationFetcher, createOffsetPaginationFetcher, createOffsetPaginationHasMoreFetcher, fetcher, loadQuery, setFilterValue };
